!!!!!!!!!!!!!!!  BEGIN of code inserted by the arcs compiler  !!!!!!!!!!!!!!!!
      implicit none
      character*(15) whoami
      parameter(whoami='estimate_wobble')
      include "INPUT/set_time_range.net.par"
      include "INPUT/set_nset.nset.par"
      real ast_pos_s(3,net)
      double precision ast_pos(3,net)
      integer nsetused
      double precision scpos(3,net,nset)
      double precision startet
      double precision det
      character*(320) filename
      real ssc_pos(3,net)
      double precision sc_pos(3,net)
      double precision var
!!!!!!!!!!!!!!!!  END of code inserted by the arcs compiler  !!!!!!!!!!!!!!!!!

!****************************************************************************!
!- Table of contents
!#TOC
!****************************************************************************!
!- 1 Declarations
!-   1.1 Variables
!- 2 Set orbital perid
!- 3 Loop through sets
!-   3.1 Make smoothed version
!-   3.2 Subtract from original
!-   3.3 Estimate wobble factor
!-   3.4 Update variance (off)
!-   3.5 Print mean error (off)
!- 4 Loop through wobble factors (off)
!-   4.1 Loop through time points
!-       4.1.1 Subtract wobble from SC position
!-       4.1.2 Add contribution to second moment
!-   4.2 Print second moment (off)
!- 5 Another approach, based on the vector component
!-   5.1 Compute norm of the wobble
!-   5.2 Invert direction and scale wobble with norm
!-   5.3 Check norm of the wobble (off, variable norm needed later)
!-   5.4 Compute scalar product with estimated trajectory
!-   5.5 Update error
!- 6 Print mean error
!- 7 Put to output for DX

!****************************************************************************!
!- 1
!# Declarations
!****************************************************************************!

!============================================================================!
!- 1.1
!## Variables
!============================================================================!
      double precision   delta
      double precision   dhelp
      double precision   fac
      integer            idim
      integer            iet
      integer            ifac
      integer            iset
      integer            jet
      double precision   m2
      double precision   m2_1
      double precision   m2_2
      integer            n
      double precision   norm
      double precision   norm1
      integer            np
      double precision   pos(3)
      double precision   pos1(3)
      double precision   projected(net)
      double precision   smoothed(3,net)
      double precision   sprod
!      double precision   var
      double precision   vdist
      double precision   vnorm

!!!!!!!!!!!!!!!  BEGIN of code inserted by the arcs compiler  !!!!!!!!!!!!!!!!
      open( 42,
     &file = 'INPUT/wobble'
     &//'.ast_pos.d',
     &form = 'UNFORMATTED' )
      read(42) ast_pos_s
      read(42) ast_pos
      close( 42 )
      open( 42,
     &file = 'estimate_trajectory'
     &//'.nsetused.d',
     &form = 'UNFORMATTED' )
      read(42) nsetused
      close( 42 )
      open( 42,
     &file = 'estimate_trajectory'
     &//'.sc_pos_est.d',
     &form = 'UNFORMATTED' )
      read(42) scpos
      close( 42 )
      open( 42,
     &file = 'INPUT/set_time_range'
     &//'.time_range.d',
     &form = 'UNFORMATTED' )
      read(42) startet
      read(42) det
      close( 42 )
!!!!!!!!!!!!!!!!  END of code inserted by the arcs compiler  !!!!!!!!!!!!!!!!!

!****************************************************************************!
!- 2
!# Set orbital perid
!****************************************************************************!
      np = nint( 42912d0 / det )
      print*, 'Orbital period is', np, ' time steps.'

!****************************************************************************!
!- 3
!# Loop through sets
!****************************************************************************!
      var = 0d0
      do iset = 1, nsetused
         print*, 'Set', iset

!============================================================================!
!- 3.1
!## Make smoothed version
!============================================================================!
         do idim = 1, 3
            do iet = 1, net - np
               smoothed(idim,iet) = 0d0
               do jet = 1, np
                  smoothed(idim,iet) = smoothed(idim,iet) +
     &                              scpos(idim,iet+jet-1,iset)
               end do
               smoothed(idim,iet) = smoothed(idim,iet) / np
            end do
            do iet = 1 + net - np, net
               smoothed(idim,iet) = smoothed(idim,net-np)
            end do
         end do

!============================================================================!
!- 3.2
!## Subtract from original
!============================================================================!
         do idim = 1, 3
            do iet = 1, net
               if ( iet .gt. np/2 .and. iet .le. net - np/2 ) then
                  jet = iet - np/2
                  if ( mod( np, 2 ) .ne. 0 ) then
                     scpos(idim,iet,iset) = scpos(idim,iet,iset) -
     &                                   smoothed(idim,jet)
                  else
                     scpos(idim,iet,iset) = scpos(idim,iet,iset) -
     &               ( smoothed(idim,jet) + smoothed(idim,jet+1) ) / 2d0
                  end if
               else
                  scpos(idim,iet,iset) = 0d0
               end if
            end do
         end do

!============================================================================!
!- 3.3
!## Estimate wobble factor
!============================================================================!
         n = 0
         fac = 0d0
         do iet = np/2 + 1, net - np/2
            do idim = 1, 3
               n = n + 1
               delta = -scpos(idim,iet,iset) / ast_pos(idim,iet) - fac
               fac = fac + delta / n
            end do
         end do
!         print*, 'Estimated wobble factor is', real( fac )

!============================================================================!
!- 3.4
!## Update variance (off)
!============================================================================!
!         delta = ( fac - 1d0 )**2 - var
!         var = var + delta / iset

      end do

!============================================================================!
!- 3.5
!## Print mean error (off)
!============================================================================!
!      print*, 'Mean error:', real( sqrt( var ) * 100d0 ), '%'

!****************************************************************************!
!- 4
!# Loop through wobble factors (off)
!****************************************************************************!
      m2_2 = -2d0
      m2_1 = -1d0
      do ifac = -100, -100 ! 100
         fac = 10d0**(ifac/100d0)
!         print*, ifac, fac

!============================================================================!
!- 4.1
!## Loop through time points
!============================================================================!
         m2 = 0d0
         n = 0
         do iet = 1, net

!----------------------------------------------------------------------------!
!- 4.1.1
!### Subtract wobble from SC position
!----------------------------------------------------------------------------!
            do idim = 1, 3
               pos(idim) = scpos(idim,iet,iset) +
     &                     fac * ast_pos(idim,iet)
            end do

!----------------------------------------------------------------------------!
!- 4.1.2
!### Add contribution to second moment
!----------------------------------------------------------------------------!
            if ( iet .ge. 2 ) then
               n = n + 1
               delta = vdist( pos, pos1 )**2
               m2 = m2 + delta / n
            end if
            call vequ( pos, pos1 )

         end do

!============================================================================!
!- 4.2
!## Print second moment (off)
!============================================================================!
         if ( m2_2 .gt. m2_1 .and. m2_1 .lt. m2 ) then
!            print*, real(10d0**((ifac-2)/100d0)), m2_2
!            print*, real(10d0**((ifac-1)/100d0)), m2_1
!            print*, real(fac), m2
         end if
         m2_2 = m2_1
         m2_1 = m2

      end do

!****************************************************************************!
!- 5
!# Another approach, based on the vector component
!****************************************************************************!

!============================================================================!
!- 5.1
!## Compute norm of the wobble
!============================================================================!
      norm = 0d0
      do iet = 1, net
         do idim = 1,3
!            norm = norm + ast_pos(idim,iet)**2
            pos(idim) = ast_pos(idim,iet)
         end do
!         print*, vnorm( pos )
         delta = vnorm( pos ) - norm
         norm = norm + delta / iet
      end do
!      norm = sqrt( norm / iet )
      print*, 'norm =', norm

!============================================================================!
!- 5.2
!## Invert direction and scale wobble with norm
!============================================================================!
      do iet = 1, net
         do idim = 1,3
            ast_pos(idim,iet) = -1d0 * ast_pos(idim,iet) / norm
         end do
      end do

!============================================================================!
!- 5.3
!## Check norm of the wobble (off, variable norm needed later)
!============================================================================!
!      norm = 0d0
!      do iet = 1, net
!!         norm1 = 0d0
!         do idim = 1,3
!!            norm1 = norm1 + ast_pos(idim,iet)**2
!            pos(idim) = ast_pos(idim,iet)
!         end do
!!         norm = norm + sqrt( norm1 )
!         delta = vnorm( pos ) - norm
!         norm = norm + delta / iet
!      end do
!!      norm = ( norm / iet )
!      print*, 'norm =', norm

      do iset = 1, nsetused

!============================================================================!
!- 5.4
!## Compute scalar product with estimated trajectory
!============================================================================!
         sprod = 0d0
         n = 0
         do iet = np/2 + 1, net - np/2
            dhelp = 0d0
            do idim = 1,3
               dhelp = dhelp + scpos(idim,iet,iset) *
     &                         ast_pos(idim,iet)
            end do
            projected(iet) = dhelp
            n = n + 1
            delta = dhelp - sprod
            sprod = sprod + delta / n
         end do
         print*, iset, real( sprod )

!============================================================================!
!- 5.5
!## Update error
!============================================================================!
         delta = ( sprod - norm )**2 - var
         var = var + delta / iset

      end do

!****************************************************************************!
!- 6
!# Print mean error
!****************************************************************************!
      var = sqrt( var ) / norm
      print*, 'Mean error [%]:', real( 100d0 * var )

!****************************************************************************!
!- 7
!# Put to output for DX
!****************************************************************************!
      do iet = 1, net
         do idim = 1, 3
!            sc_pos(idim,iet) = scpos(idim,iet,1)
!            sc_pos(idim,iet) = smoothed(idim,iet)
!            sc_pos(idim,iet) = ast_pos(idim,iet)
            sc_pos(idim,iet) = projected(iet)
            ssc_pos(idim,iet) = real( sc_pos(idim,iet) )
         end do
      end do
      filename = whoami//'.sc_position'

!!!!!!!!!!!!!!!  BEGIN of code inserted by the arcs compiler  !!!!!!!!!!!!!!!!
      open(42,file=
     &'estimate_wobble.filename.d',
     &form = 'UNFORMATTED' )
      write(42) filename
      close(42)
      open(42,file=
     &'estimate_wobble.sc_position.d',
     &form = 'UNFORMATTED' )
      write(42) ssc_pos
      write(42) sc_pos
      close(42)
      open(42,file=
     &'estimate_wobble.wobble_err.d',
     &form = 'UNFORMATTED' )
      write(42) var
      close(42)
      end
!!!!!!!!!!!!!!!!  END of code inserted by the arcs compiler  !!!!!!!!!!!!!!!!!
